==> fred_regal-1.3.0-py3-none-any.whl: regal/nsset.py <==
"""Nsset client for registry service."""
from datetime import datetime
from ipaddress import IPv4Address, IPv6Address, ip_address
from typing import Any, Dict, List, Optional, Sequence, Union, cast

import fred_api.registry.nsset.service_nsset_grpc_pb2_grpc
import grpc
from fred_api.registry.nsset.nsset_history_types_pb2 import NssetHistoryReply, NssetHistoryRequest
from fred_api.registry.nsset.nsset_info_types_pb2 import (
    CheckDnsHostRequest,
    DnsHost as DnsHostMessage,
    NssetIdReply,
    NssetIdRequest,
    NssetInfoReply,
    NssetInfoRequest,
)
from fred_api.registry.nsset.nsset_state_history_types_pb2 import NssetStateHistoryReply, NssetStateHistoryRequest
from fred_api.registry.nsset.nsset_state_types_pb2 import NssetStateReply, NssetStateRequest
from fred_types import BaseModel, ContactRef, NssetHistoryId, NssetId

from .utils import _idna_encode

try:
    from pydantic.v1 import root_validator
except ImportError:  # pragma: no cover
    from pydantic import root_validator  # type: ignore[no-redef]

from .common import ObjectEvents, RegistryDecoder, _set_sponsoring_registrar
from .exceptions import InvalidHistoryInterval, NssetDoesNotExist
from .object import RegistryObjectClient


class DnsHost(BaseModel):
    """Dns host info."""

    fqdn: str
    ip_addresses: List[Union[IPv4Address, IPv6Address]]


class Nsset(BaseModel):
    """Nsset info."""

    nsset_id: NssetId
    nsset_handle: str
    nsset_history_id: Optional[NssetHistoryId] = None
    sponsoring_registrar: Optional[str] = None
    dns_hosts: Sequence[DnsHost] = []
    technical_contacts: Sequence[ContactRef] = []
    technical_check_level: Optional[int] = None
    auth_info: Optional[str] = None
    events: Optional[ObjectEvents] = None

    _set_sponsoring_registrar = root_validator(pre=True, allow_reuse=True)(_set_sponsoring_registrar)


class NssetDecoder(RegistryDecoder):
    """Nsset service decoder."""

    def __init__(self) -> None:
        super().__init__()
        self.set_decoder(DnsHostMessage, self._decode_dns_host)
        self.set_exception_decoder(NssetIdReply.Exception.NssetDoesNotExist, NssetDoesNotExist)
        self.set_exception_decoder(NssetInfoReply.Exception.NssetDoesNotExist, NssetDoesNotExist)
        self.set_exception_decoder(NssetHistoryReply.Exception.NssetDoesNotExist, NssetDoesNotExist)
        self.set_exception_decoder(NssetHistoryReply.Exception.InvalidHistoryInterval, InvalidHistoryInterval)
        self.set_exception_decoder(NssetStateReply.Exception.NssetDoesNotExist, NssetDoesNotExist)
        self.set_exception_decoder(NssetStateHistoryReply.Exception.NssetDoesNotExist, NssetDoesNotExist)
        self.set_exception_decoder(NssetStateHistoryReply.Exception.InvalidHistoryInterval, InvalidHistoryInterval)

    def _decode_dns_host(self, value: DnsHostMessage) -> DnsHost:
        """Decode DNS host."""
        return DnsHost(
            fqdn=self.decode(value.fqdn),
            ip_addresses=[ip_address(address) for address in self.decode(value.ip_addresses)],
        )


class NssetClient(RegistryObjectClient[NssetId, NssetHistoryId]):
    """Nsset service gRPC client."""

    service = "Nsset"
    object_id = "nsset_id"
    object_history_id = "nsset_history_id"
    object_handle = "nsset_handle"

    def __init__(self, netloc: str, credentials: Optional[grpc.ChannelCredentials] = None) -> None:
        """Initialize nsset client instance.

        Args:
            netloc: Netloc of nsset service server.
            credentials: Optional credentials.
        """
        super().__init__(
            netloc,
            decoder=NssetDecoder(),
            grpc_modules=[fred_api.registry.nsset.service_nsset_grpc_pb2_grpc],
            credentials=credentials,
        )

    async def get_nsset_id(self, handle: str) -> NssetId:
        """Get nsset id.

        Args:
            handle: Nsset handle.

        Raises:
            NssetDoesNotExist: If nsset does not exist.
        """
        return NssetId(await self.get_object_id(request=NssetIdRequest(), method="get_nsset_id", handle=handle))

    async def get_nsset_info(self, nsset_id: NssetId, history_id: Optional[NssetHistoryId] = None) -> Nsset:
        """Get nsset info.

        Args:
            nsset_id: Nsset ID.
            history_id: Nsset history ID.

        Raises:
            NssetDoesNotExist: If nsset does not exist.
        """
        result = await self.get_object_info(
            request=NssetInfoRequest(),
            method="get_nsset_info",
            object_id=nsset_id,
            history_id=history_id,
        )
        result.pop("technical_contacts_old", None)
        return Nsset(**result)

    async def get_nsset_state(self, nsset_id: NssetId) -> Dict[str, bool]:
        """Get nsset state consisting of state flags and their boolean values.

        Raises:
            NssetDoesNotExist: If nsset does not exist.

        Args:
            nsset_id: Nsset ID.
        """
        return await self.get_object_state(request=NssetStateRequest(), method="get_nsset_state", object_id=nsset_id)

    async def get_nsset_history(
        self,
        nsset_id: NssetId,
        start: Optional[Union[NssetHistoryId, datetime]] = None,
        end: Optional[Union[NssetHistoryId, datetime]] = None,
    ) -> Dict[str, Any]:
        """Get nsset history timeline.

        Args:
            nsset_id: Nsset ID.
            start: Datetime or history_id of history interval start.
            end: Datetime or history_id of history interval end.

        Raises:
            NssetDoesNotExist: If nsset does not exist.
            InvalidHistoryInterval: In case of invalid history interval.
        """
        return await self.get_object_history(
            request=NssetHistoryRequest(),
            method="get_nsset_history",
            object_id=nsset_id,
            start=start,
            end=end,
        )

    async def get_nsset_state_history(
        self,
        nsset_id: NssetId,
        start: Optional[Union[NssetHistoryId, datetime]] = None,
        end: Optional[Union[NssetHistoryId, datetime]] = None,
    ) -> Dict[str, Any]:
        """Get nsset state history timeline.

        Args:
            nsset_id: Nsset ID.
            start: Datetime or history_id of history interval start.
            end: Datetime or history_id of history interval end.

        Raises:
            NssetDoesNotExist: If nsset does not exist.
            InvalidHistoryInterval: In case of invalid history interval.
        """
        return await self.get_object_state_history(
            request=NssetStateHistoryRequest(),
            method="get_nsset_state_history",
            object_id=nsset_id,
            start=start,
            end=end,
        )

    async def check_dns_host(self, fqdn: str) -> bool:
        """Return whether DNS host exists.

        This method returns `True` if there is at least one registered nsset containing nameserver with `fqdn`.

        Args:
            fqdn: DNS host's fqdn.
        """
        request = CheckDnsHostRequest(fqdn=_idna_encode(fqdn))
        reply = await self.call(self.service, "check_dns_host", request)
        return cast(bool, reply)

==> fred_regal-1.5.0-py3-none-any.whl: regal/nsset.py <==
"""Nsset client for registry service."""
from datetime import datetime
from ipaddress import IPv4Address, IPv6Address, ip_address
from typing import Any, Dict, List, Optional, Sequence, Union, cast

import fred_api.registry.nsset.service_nsset_grpc_pb2_grpc
import grpc
from fred_api.registry.nsset.nsset_history_types_pb2 import NssetHistoryReply, NssetHistoryRequest
from fred_api.registry.nsset.nsset_info_types_pb2 import (
    CheckDnsHostRequest,
    DnsHost as DnsHostMessage,
    NssetIdReply,
    NssetIdRequest,
    NssetInfoReply,
    NssetInfoRequest,
)
from fred_api.registry.nsset.nsset_state_history_types_pb2 import NssetStateHistoryReply, NssetStateHistoryRequest
from fred_api.registry.nsset.nsset_state_types_pb2 import NssetStateReply, NssetStateRequest
from fred_types import BaseModel, ContactRef, NssetHistoryId, NssetId

from .utils import _idna_encode

try:
    from pydantic.v1 import root_validator
except ImportError:  # pragma: no cover
    from pydantic import root_validator  # type: ignore[no-redef]

from .common import ObjectEvents, RegistryDecoder, _set_sponsoring_registrar
from .exceptions import InvalidHistoryInterval, NssetDoesNotExist
from .object import ObjectStateFlag, RegistryObjectClient


class DnsHost(BaseModel):
    """Dns host info."""

    fqdn: str
    ip_addresses: List[Union[IPv4Address, IPv6Address]]


class Nsset(BaseModel):
    """Nsset info."""

    nsset_id: NssetId
    nsset_handle: str
    nsset_history_id: Optional[NssetHistoryId] = None
    sponsoring_registrar: Optional[str] = None
    dns_hosts: Sequence[DnsHost] = []
    technical_contacts: Sequence[ContactRef] = []
    technical_check_level: Optional[int] = None
    auth_info: Optional[str] = None
    events: Optional[ObjectEvents] = None

    _set_sponsoring_registrar = root_validator(pre=True, allow_reuse=True)(_set_sponsoring_registrar)


class NssetDecoder(RegistryDecoder):
    """Nsset service decoder."""

    def __init__(self) -> None:
        super().__init__()
        self.set_decoder(DnsHostMessage, self._decode_dns_host)
        self.set_exception_decoder(NssetIdReply.Exception.NssetDoesNotExist, NssetDoesNotExist)
        self.set_exception_decoder(NssetInfoReply.Exception.NssetDoesNotExist, NssetDoesNotExist)
        self.set_exception_decoder(NssetHistoryReply.Exception.NssetDoesNotExist, NssetDoesNotExist)
        self.set_exception_decoder(NssetHistoryReply.Exception.InvalidHistoryInterval, InvalidHistoryInterval)
        self.set_exception_decoder(NssetStateReply.Exception.NssetDoesNotExist, NssetDoesNotExist)
        self.set_exception_decoder(NssetStateHistoryReply.Exception.NssetDoesNotExist, NssetDoesNotExist)
        self.set_exception_decoder(NssetStateHistoryReply.Exception.InvalidHistoryInterval, InvalidHistoryInterval)

    def _decode_dns_host(self, value: DnsHostMessage) -> DnsHost:
        """Decode DNS host."""
        return DnsHost(
            fqdn=self.decode(value.fqdn),
            ip_addresses=[ip_address(address) for address in self.decode(value.ip_addresses)],
        )


class NssetClient(RegistryObjectClient[NssetId, NssetHistoryId]):
    """Nsset service gRPC client."""

    service = "Nsset"
    object_id = "nsset_id"
    object_history_id = "nsset_history_id"
    object_handle = "nsset_handle"

    def __init__(self, netloc: str, credentials: Optional[grpc.ChannelCredentials] = None) -> None:
        """Initialize nsset client instance.

        Args:
            netloc: Netloc of nsset service server.
            credentials: Optional credentials.
        """
        super().__init__(
            netloc,
            decoder=NssetDecoder(),
            grpc_modules=[fred_api.registry.nsset.service_nsset_grpc_pb2_grpc],
            credentials=credentials,
        )

    async def get_nsset_id(self, handle: str) -> NssetId:
        """Get nsset id.

        Args:
            handle: Nsset handle.

        Raises:
            NssetDoesNotExist: If nsset does not exist.
        """
        return NssetId(await self.get_object_id(request=NssetIdRequest(), method="get_nsset_id", handle=handle))

    async def get_nsset_info(self, nsset_id: NssetId, history_id: Optional[NssetHistoryId] = None) -> Nsset:
        """Get nsset info.

        Args:
            nsset_id: Nsset ID.
            history_id: Nsset history ID.

        Raises:
            NssetDoesNotExist: If nsset does not exist.
        """
        result = await self.get_object_info(
            request=NssetInfoRequest(),
            method="get_nsset_info",
            object_id=nsset_id,
            history_id=history_id,
        )
        result.pop("technical_contacts_old", None)
        return Nsset(**result)

    async def get_nsset_state(self, nsset_id: NssetId, *, internal: bool = True) -> Dict[str, bool]:
        """Get nsset state consisting of state flags and their boolean values.

        Args:
            nsset_id: Nsset ID.
            internal: Whether to include internal states in the result.

        Raises:
            NssetDoesNotExist: If nsset does not exist.
        """
        return await self.get_object_state(
            request=NssetStateRequest(), method="get_nsset_state", object_id=nsset_id, internal=internal
        )

    async def get_nsset_history(
        self,
        nsset_id: NssetId,
        start: Optional[Union[NssetHistoryId, datetime]] = None,
        end: Optional[Union[NssetHistoryId, datetime]] = None,
    ) -> Dict[str, Any]:
        """Get nsset history timeline.

        Args:
            nsset_id: Nsset ID.
            start: Datetime or history_id of history interval start.
            end: Datetime or history_id of history interval end.

        Raises:
            NssetDoesNotExist: If nsset does not exist.
            InvalidHistoryInterval: In case of invalid history interval.
        """
        return await self.get_object_history(
            request=NssetHistoryRequest(),
            method="get_nsset_history",
            object_id=nsset_id,
            start=start,
            end=end,
        )

    async def get_nsset_state_history(
        self,
        nsset_id: NssetId,
        start: Optional[Union[NssetHistoryId, datetime]] = None,
        end: Optional[Union[NssetHistoryId, datetime]] = None,
    ) -> Dict[str, Any]:
        """Get nsset state history timeline.

        Args:
            nsset_id: Nsset ID.
            start: Datetime or history_id of history interval start.
            end: Datetime or history_id of history interval end.

        Raises:
            NssetDoesNotExist: If nsset does not exist.
            InvalidHistoryInterval: In case of invalid history interval.
        """
        return await self.get_object_state_history(
            request=NssetStateHistoryRequest(),
            method="get_nsset_state_history",
            object_id=nsset_id,
            start=start,
            end=end,
        )

    async def get_state_flags(self) -> Dict[str, ObjectStateFlag]:
        """Get object state flags and their properties."""
        return await self.get_object_state_flags(method="get_nsset_state_flags")

    async def check_dns_host(self, fqdn: str) -> bool:
        """Return whether DNS host exists.

        This method returns `True` if there is at least one registered nsset containing nameserver with `fqdn`.

        Args:
            fqdn: DNS host's fqdn.
        """
        request = CheckDnsHostRequest(fqdn=_idna_encode(fqdn))
        reply = await self.call(self.service, "check_dns_host", request)
        return cast(bool, reply)
